@@ -1,27 +1,19 @@
-# This file is part of Androguard.
-#
-# Copyright (C) 2012/2013, Anthony Desnos <desnos at t0t0.fr>
-# All rights reserved.
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#      http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS-IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+from __future__ import print_function
+from __future__ import absolute_import
 
+from builtins import str
+from builtins import range
+from builtins import object
 import hashlib
 from xml.sax.saxutils import escape
 from struct import unpack, pack
 import textwrap
 
 import json
-from androguard.core.androconf import warning, error, CONF, enable_colors, remove_colors, save_colors, color_range
+from .androconf import CONF, enable_colors, remove_colors, save_colors, color_range
+import logging
+
+log = logging.getLogger("androguard.bytecode")
 
 
 def disable_print_colors():
@@ -36,12 +28,8 @@ def enable_print_colors(colors):
 
 # Handle exit message
 def Exit(msg):
-    warning("Error : " + msg)
-    raise("oops")
-
-
-def Warning(msg):
-    warning(msg)
+    log.warning("Error : " + msg)
+    raise Exception("oops")
 
 
 def _PrintBanner():
@@ -51,7 +39,7 @@ def _PrintBanner():
 
 def _PrintSubBanner(title=None):
     print_fct = CONF["PRINT_FCT"]
-    if title == None:
+    if title is None:
         print_fct("#" * 20 + "\n")
     else:
         print_fct("#" * 10 + " " + title + "\n")
@@ -61,12 +49,10 @@ def _PrintNote(note, tab=0):
     print_fct = CONF["PRINT_FCT"]
     note_color = CONF["COLORS"]["NOTE"]
     normal_color = CONF["COLORS"]["NORMAL"]
-    print_fct("\t" * tab + "%s# %s%s" %
-              (note_color, note, normal_color) + "\n")
+    print_fct("\t" * tab + "%s# %s%s" % (note_color, note, normal_color) + "\n")
+
 
 # Print arg into a correct format
-
-
 def _Print(name, arg):
     buff = name + " "
 
@@ -88,27 +74,31 @@ def PrettyShowEx(exceptions):
     if len(exceptions) > 0:
         CONF["PRINT_FCT"]("Exceptions:\n")
         for i in exceptions:
-            CONF["PRINT_FCT"]("\t%s%s%s\n" % (
-                CONF["COLORS"]["EXCEPTION"], i.show_buff(), CONF["COLORS"]["NORMAL"]))
+            CONF["PRINT_FCT"]("\t%s%s%s\n" %
+                              (CONF["COLORS"]["EXCEPTION"], i.show_buff(),
+                               CONF["COLORS"]["NORMAL"]))
 
 
 def _PrintXRef(tag, items):
     print_fct = CONF["PRINT_FCT"]
     for i in items:
-        print_fct("%s: %s %s %s %s\n" % (tag, i[0].get_class_name(), i[0].get_name(), i[
-                  0].get_descriptor(), ' '.join("%x" % j.get_idx() for j in i[1])))
+        print_fct("%s: %s %s %s %s\n" %
+                  (tag, i[0].get_class_name(), i[0].get_name(),
+                   i[0].get_descriptor(), ' '.join("%x" % j.get_idx()
+                                                   for j in i[1])))
 
 
 def _PrintDRef(tag, items):
     print_fct = CONF["PRINT_FCT"]
     for i in items:
-        print_fct("%s: %s %s %s %s\n" % (tag, i[0].get_class_name(), i[
-                  0].get_name(), i[0].get_descriptor(), ' '.join("%x" % j for j in i[1])))
+        print_fct("%s: %s %s %s %s\n" %
+                  (tag, i[0].get_class_name(), i[0].get_name(),
+                   i[0].get_descriptor(), ' '.join("%x" % j for j in i[1])))
 
 
 def _PrintDefault(msg):
     print_fct = CONF["PRINT_FCT"]
-    print_fct(msg)
+    print_fct(msg.encode('unicode_escape').decode("ASCII"))
 
 
 def PrettyShow(m_a, basic_blocks, notes={}):
@@ -130,20 +120,22 @@ def PrettyShow(m_a, basic_blocks, notes={}):
 
     for i in basic_blocks:
         print_fct("%s%s%s : \n" % (bb_color, i.get_name(), normal_color))
-        instructions = i.get_instructions()
+        instructions = list(i.get_instructions())
         for ins in instructions:
             if nb in notes:
                 for note in notes[nb]:
                     _PrintNote(note, 1)
 
-            print_fct("\t%s%-3d%s(%s%08x%s) " % (offset_color, nb,
-                                                 normal_color, offset_addr_color, idx, normal_color))
+            print_fct("\t%s%-3d%s(%s%08x%s) " %
+                      (offset_color, nb, normal_color, offset_addr_color, idx,
+                       normal_color))
             print_fct("%s%-20s%s" %
                       (instruction_name_color, ins.get_name(), normal_color))
 
             operands = ins.get_operands()
-            print_fct("%s" % ", ".join(
-                m_a.get_vm().colorize_operands(operands, colors)))
+            print_fct(
+                "%s" %
+                ", ".join(m_a.get_vm().colorize_operands(operands, colors)))
 
             op_value = ins.get_op_value()
             if ins == instructions[-1] and i.childs:
@@ -152,19 +144,24 @@ def PrettyShow(m_a, basic_blocks, notes={}):
                 # packed/sparse-switch
                 if (op_value == 0x2b or op_value == 0x2c) and len(i.childs) > 1:
                     values = i.get_special_ins(idx).get_values()
-                    print_fct("%s[ D:%s%s " % (branch_false_color,
-                                               i.childs[0][2].get_name(), branch_color))
-                    print_fct(' '.join("%d:%s" % (values[j], i.childs[
-                              j + 1][2].get_name()) for j in range(0, len(i.childs) - 1)) + " ]%s" % normal_color)
+                    print_fct("%s[ D:%s%s " %
+                              (branch_false_color, i.childs[0][2].get_name(),
+                               branch_color))
+                    print_fct(' '.join("%d:%s" % (
+                        values[j], i.childs[j + 1][2].get_name()) for j in
+                                       range(0, len(i.childs) - 1)) + " ]%s" %
+                              normal_color)
                 else:
                     if len(i.childs) == 2:
-                        print_fct("%s[ %s%s " % (branch_false_color, i.childs[
-                                  0][2].get_name(), branch_true_color))
-                        print_fct(' '.join("%s" % c[2].get_name() for c in i.childs[
-                                  1:]) + " ]%s" % normal_color)
+                        print_fct("%s[ %s%s " % (branch_false_color,
+                                                 i.childs[0][2].get_name(),
+                                                 branch_true_color))
+                        print_fct(' '.join("%s" % c[2].get_name(
+                        ) for c in i.childs[1:]) + " ]%s" % normal_color)
                     else:
-                        print_fct("%s[ " % branch_color + ' '.join("%s" % c[2].get_name()
-                                                                   for c in i.childs) + " ]%s" % normal_color)
+                        print_fct("%s[ " % branch_color + ' '.join(
+                            "%s" % c[2].get_name() for c in i.childs) + " ]%s" %
+                                  normal_color)
 
             idx += ins.get_length()
             nb += 1
@@ -172,8 +169,9 @@ def PrettyShow(m_a, basic_blocks, notes={}):
             print_fct("\n")
 
         if i.get_exception_analysis():
-            print_fct("\t%s%s%s\n" % (exception_color,
-                                      i.exception_analysis.show_buff(), normal_color))
+            print_fct("\t%s%s%s\n" %
+                      (exception_color, i.exception_analysis.show_buff(),
+                       normal_color))
 
         print_fct("\n")
 
@@ -188,27 +186,26 @@ def method2dot(mx, colors={}):
         @rtype : dot format buffer (it is a subgraph (dict))
     """
 
-    colors = colors or {"true_branch": "green",
-                        "false_branch": "red",
-                        "default_branch": "purple",
-                        "jump_branch": "blue",
-                        "bg_idx": "lightgray",
-                        "idx": "blue",
-                        "bg_start_idx": "yellow",
-                        "bg_instruction": "lightgray",
-                        "instruction_name": "black",
-                        "instructions_operands": "yellow",
-
-                        "raw": "red",
-                        "string": "red",
-                        "literal": "green",
-                        "offset": "#4000FF",
-                        "method": "#DF3A01",
-                        "field": "#088A08",
-                        "type": "#0000FF",
-
-                        "registers_range": ("#999933", "#6666FF")
-                        }
+    colors = colors or {
+        "true_branch": "green",
+        "false_branch": "red",
+        "default_branch": "purple",
+        "jump_branch": "blue",
+        "bg_idx": "lightgray",
+        "idx": "blue",
+        "bg_start_idx": "yellow",
+        "bg_instruction": "lightgray",
+        "instruction_name": "black",
+        "instructions_operands": "yellow",
+        "raw": "red",
+        "string": "red",
+        "literal": "green",
+        "offset": "#4000FF",
+        "method": "#DF3A01",
+        "field": "#088A08",
+        "type": "#0000FF",
+        "registers_range": ("#999933", "#6666FF")
+    }
 
     node_tpl = "\nstruct_%s [label=<\n<TABLE BORDER=\"0\" CELLBORDER=\"0\" CELLSPACING=\"3\">\n%s</TABLE>>];\n"
     label_tpl = "<TR><TD ALIGN=\"LEFT\" BGCOLOR=\"%s\"> <FONT FACE=\"Times-Bold\" color=\"%s\">%x</FONT> </TD><TD ALIGN=\"LEFT\" BGCOLOR=\"%s\"> <FONT FACE=\"Times-Bold\" color=\"%s\">%s </FONT> %s </TD></TR>\n"
@@ -218,8 +215,9 @@ def method2dot(mx, colors={}):
     blocks_html = ""
 
     method = mx.get_method()
-    sha256 = hashlib.sha256("%s%s%s" % (mx.get_method().get_class_name(
-    ), mx.get_method().get_name(), mx.get_method().get_descriptor())).hexdigest()
+    sha256 = hashlib.sha256(bytearray("%s%s%s" % (
+        mx.get_method().get_class_name(), mx.get_method().get_name(),
+        mx.get_method().get_descriptor()), "UTF-8")).hexdigest()
 
     registers = {}
     if method.get_code():
@@ -245,24 +243,28 @@ def method2dot(mx, colors={}):
 
     for DVMBasicMethodBlock in mx.basic_blocks.gets():
         ins_idx = DVMBasicMethodBlock.start
-        block_id = hashlib.md5(
-            sha256 + DVMBasicMethodBlock.get_name()).hexdigest()
+        block_id = hashlib.md5(bytearray(sha256 + DVMBasicMethodBlock.get_name(
+        ), "UTF-8")).hexdigest()
 
         content = link_tpl % 'header'
 
         for DVMBasicMethodBlockInstruction in DVMBasicMethodBlock.get_instructions():
-            if DVMBasicMethodBlockInstruction.get_op_value() == 0x2b or DVMBasicMethodBlockInstruction.get_op_value() == 0x2c:
-                new_links.append(
-                    (DVMBasicMethodBlock, ins_idx, DVMBasicMethodBlockInstruction.get_ref_off() * 2 + ins_idx))
+            if DVMBasicMethodBlockInstruction.get_op_value(
+            ) == 0x2b or DVMBasicMethodBlockInstruction.get_op_value() == 0x2c:
+                new_links.append((DVMBasicMethodBlock, ins_idx,
+                                  DVMBasicMethodBlockInstruction.get_ref_off(
+                                  ) * 2 + ins_idx))
             elif DVMBasicMethodBlockInstruction.get_op_value() == 0x26:
-                new_links.append(
-                    (DVMBasicMethodBlock, ins_idx, DVMBasicMethodBlockInstruction.get_ref_off() * 2 + ins_idx))
+                new_links.append((DVMBasicMethodBlock, ins_idx,
+                                  DVMBasicMethodBlockInstruction.get_ref_off(
+                                  ) * 2 + ins_idx))
 
             operands = DVMBasicMethodBlockInstruction.get_operands(ins_idx)
             output = ", ".join(mx.get_vm().get_operand_html(
                 i, registers, colors, escape, textwrap.wrap) for i in operands)
 
-            formatted_operands = DVMBasicMethodBlockInstruction.get_formatted_operands()
+            formatted_operands = DVMBasicMethodBlockInstruction.get_formatted_operands(
+            )
             if formatted_operands:
                 output += " ; %s" % str(formatted_operands)
 
@@ -270,13 +272,10 @@ def method2dot(mx, colors={}):
             if ins_idx == 0 and "bg_start_idx" in colors:
                 bg_idx = colors["bg_start_idx"]
 
-            content += label_tpl % (bg_idx,
-                                    colors["idx"],
-                                    ins_idx,
-                                    colors["bg_instruction"],
-                                    colors["instruction_name"],
-                                    DVMBasicMethodBlockInstruction.get_name(),
-                                    output)
+            content += label_tpl % (
+                bg_idx, colors["idx"], ins_idx, colors["bg_instruction"],
+                colors["instruction_name"],
+                DVMBasicMethodBlockInstruction.get_name(), output)
 
             ins_idx += DVMBasicMethodBlockInstruction.get_length()
             last_instru = DVMBasicMethodBlockInstruction
@@ -294,7 +293,9 @@ def method2dot(mx, colors={}):
             val = colors["jump_branch"]
 
         values = None
-        if (last_instru.get_op_value() == 0x2b or last_instru.get_op_value() == 0x2c) and len(DVMBasicMethodBlock.childs) > 1:
+        if (last_instru.get_op_value() == 0x2b or
+                last_instru.get_op_value() == 0x2c
+           ) and len(DVMBasicMethodBlock.childs) > 1:
             val = colors["default_branch"]
             values = ["default"]
             values.extend(DVMBasicMethodBlock.get_special_ins(
@@ -308,7 +309,7 @@ def method2dot(mx, colors={}):
                 label_edge = values.pop(0)
 
             child_id = hashlib.md5(
-                sha256 + DVMBasicMethodBlockChild[-1].get_name()).hexdigest()
+                bytearray(sha256 + DVMBasicMethodBlockChild[-1].get_name(), "UTF-8")).hexdigest()
             edges_html += "struct_%s:tail -> struct_%s:header  [color=\"%s\", label=\"%s\"];\n" % (
                 block_id, child_id, val, label_edge)
             # color switch
@@ -323,7 +324,7 @@ def method2dot(mx, colors={}):
                 exception_block = exception_elem[-1]
                 if exception_block:
                     exception_id = hashlib.md5(
-                        sha256 + exception_block.get_name()).hexdigest()
+                        bytearray(sha256 + exception_block.get_name(), "UTF-8")).hexdigest()
                     edges_html += "struct_%s:tail -> struct_%s:header  [color=\"%s\", label=\"%s\"];\n" % (
                         block_id, exception_id, "black", exception_elem[0])
 
@@ -332,29 +333,28 @@ def method2dot(mx, colors={}):
         DVMBasicMethodBlockChild = mx.basic_blocks.get_basic_block(link[2])
 
         if DVMBasicMethodBlockChild:
-            block_id = hashlib.md5(
-                sha256 + DVMBasicMethodBlock.get_name()).hexdigest()
-            child_id = hashlib.md5(
-                sha256 + DVMBasicMethodBlockChild.get_name()).hexdigest()
+            block_id = hashlib.md5(bytearray(sha256 + DVMBasicMethodBlock.get_name(
+            ), "UTF-8")).hexdigest()
+            child_id = hashlib.md5(bytearray(sha256 + DVMBasicMethodBlockChild.get_name(
+            ), "UTF-8")).hexdigest()
 
             edges_html += "struct_%s:tail -> struct_%s:header  [color=\"%s\", label=\"data(0x%x) to @0x%x\", style=\"dashed\"];\n" % (
                 block_id, child_id, "yellow", link[1], link[2])
 
-    method_label = method.get_class_name() + "." + method.get_name() + \
-        "->" + method.get_descriptor()
+    method_label = method.get_class_name() + "." + method.get_name(
+    ) + "->" + method.get_descriptor()
 
     method_information = method.get_information()
     if method_information:
         method_label += "\\nLocal registers v%d ... v%d" % (
-            method_information["registers"][0], method_information["registers"][1])
+            method_information["registers"][0],
+            method_information["registers"][1])
         if "params" in method_information:
             for register, rtype in method_information["params"]:
                 method_label += "\\nparam v%d = %s" % (register, rtype)
         method_label += "\\nreturn = %s" % (method_information["return"])
 
-    return {'name': method_label,
-            'nodes': blocks_html,
-            'edges': edges_html}
+    return {'name': method_label, 'nodes': blocks_html, 'edges': edges_html}
 
 
 def method2format(output, _format="png", mx=None, raw=None):
@@ -366,10 +366,8 @@ def method2format(output, _format="png", mx=None, raw=None):
         @param mx : specify the MethodAnalysis object
         @param raw : use directly a dot raw buffer if None
     """
-    try:
-        import pydot
-    except ImportError:
-        error("module pydot not found")
+    # pydot is optional!
+    import pydot
 
     buff = "digraph {\n"
     buff += "graph [rankdir=TB]\n"
@@ -381,8 +379,8 @@ def method2format(output, _format="png", mx=None, raw=None):
         data = method2dot(mx)
 
     # subgraphs cluster
-    buff += "subgraph cluster_" + \
-        hashlib.md5(output).hexdigest() + " {\nlabel=\"%s\"\n" % data['name']
+    buff += "subgraph cluster_" + hashlib.md5(bytearray(output, "UTF-8")).hexdigest(
+    ) + " {\nlabel=\"%s\"\n" % data['name']
     buff += data['nodes']
     buff += "}\n"
 
@@ -390,7 +388,7 @@ def method2format(output, _format="png", mx=None, raw=None):
     buff += data['edges']
     buff += "}\n"
 
-    d = pydot.graph_from_dot_data(buff)
+    d = pydot.graph_from_dot_data(buff.encode("UTF-8"))
     if d:
         getattr(d, "write_" + _format.lower())(output)
 
@@ -407,7 +405,7 @@ def method2png(output, mx, raw=False):
         :type raw: string
     """
     buff = raw
-    if raw == False:
+    if not raw:
         buff = method2dot(mx)
 
     method2format(output, "png", mx, buff)
@@ -425,26 +423,20 @@ def method2jpg(output, mx, raw=False):
         :type raw: string
     """
     buff = raw
-    if raw == False:
+    if not raw:
         buff = method2dot(mx)
 
     method2format(output, "jpg", mx, buff)
 
 
 def vm2json(vm):
-    d = {}
-    d["name"] = "root"
-    d["children"] = []
+    d = {"name": "root", "children": []}
 
     for _class in vm.get_classes():
-        c_class = {}
-        c_class["name"] = _class.get_name()
-        c_class["children"] = []
+        c_class = {"name": _class.get_name(), "children": []}
 
         for method in _class.get_methods():
-            c_method = {}
-            c_method["name"] = method.get_name()
-            c_method["children"] = []
+            c_method = {"name": method.get_name(), "children": []}
 
             c_class["children"].append(c_method)
 
@@ -474,19 +466,14 @@ def method2json_undirect(mx):
     d["reports"] = reports
 
     for DVMBasicMethodBlock in mx.basic_blocks.gets():
-        cblock = {}
-
-        cblock["BasicBlockId"] = DVMBasicMethodBlock.get_name()
-        cblock["registers"] = mx.get_method().get_code().get_registers_size()
-        cblock["instructions"] = []
+        cblock = {"BasicBlockId": DVMBasicMethodBlock.get_name(),
+                  "registers": mx.get_method().get_code().get_registers_size(), "instructions": []}
 
         ins_idx = DVMBasicMethodBlock.start
         for DVMBasicMethodBlockInstruction in DVMBasicMethodBlock.get_instructions():
-            c_ins = {}
-            c_ins["idx"] = ins_idx
-            c_ins["name"] = DVMBasicMethodBlockInstruction.get_name()
-            c_ins["operands"] = DVMBasicMethodBlockInstruction.get_operands(
-                ins_idx)
+            c_ins = {"idx": ins_idx, "name": DVMBasicMethodBlockInstruction.get_name(),
+                     "operands": DVMBasicMethodBlockInstruction.get_operands(
+                         ins_idx)}
 
             cblock["instructions"].append(c_ins)
             ins_idx += DVMBasicMethodBlockInstruction.get_length()
@@ -509,21 +496,20 @@ def method2json_direct(mx):
 
     l = []
     for DVMBasicMethodBlock in mx.basic_blocks.gets():
-        for index, DVMBasicMethodBlockChild in enumerate(DVMBasicMethodBlock.childs):
-            if DVMBasicMethodBlock.get_name() == DVMBasicMethodBlockChild[-1].get_name():
+        for index, DVMBasicMethodBlockChild in enumerate(
+            DVMBasicMethodBlock.childs):
+            if DVMBasicMethodBlock.get_name(
+            ) == DVMBasicMethodBlockChild[-1].get_name():
 
                 preblock = TmpBlock(DVMBasicMethodBlock.get_name() + "-pre")
 
-                cnblock = {}
-                cnblock["BasicBlockId"] = DVMBasicMethodBlock.get_name() + \
-                    "-pre"
-                cnblock["start"] = DVMBasicMethodBlock.start
-                cnblock["notes"] = []
-
-                cnblock["Edge"] = [DVMBasicMethodBlock.get_name()]
-                cnblock["registers"] = 0
-                cnblock["instructions"] = []
-                cnblock["info_bb"] = 0
+                cnblock = {"BasicBlockId": DVMBasicMethodBlock.get_name() + "-pre",
+                           "start": DVMBasicMethodBlock.start,
+                           "notes": [],
+                           "Edge": [DVMBasicMethodBlock.get_name()],
+                           "registers": 0,
+                           "instructions": [],
+                           "info_bb": 0}
 
                 l.append(cnblock)
 
@@ -532,34 +518,29 @@ def method2json_direct(mx):
                     hooks[parent[-1].get_name()].append(preblock)
 
                     for idx, child in enumerate(parent[-1].childs):
-                        if child[-1].get_name() == DVMBasicMethodBlock.get_name():
+                        if child[-1].get_name() == DVMBasicMethodBlock.get_name(
+                        ):
                             hooks[parent[-1].get_name()].append(child[-1])
 
     for DVMBasicMethodBlock in mx.basic_blocks.gets():
-        cblock = {}
-
-        cblock["BasicBlockId"] = DVMBasicMethodBlock.get_name()
-        cblock["start"] = DVMBasicMethodBlock.start
-        cblock["notes"] = DVMBasicMethodBlock.get_notes()
-
-        cblock["registers"] = mx.get_method().get_code().get_registers_size()
-        cblock["instructions"] = []
+        cblock = {"BasicBlockId": DVMBasicMethodBlock.get_name(),
+                  "start": DVMBasicMethodBlock.start,
+                  "notes": DVMBasicMethodBlock.get_notes(),
+                  "registers": mx.get_method().get_code().get_registers_size(),
+                  "instructions": []}
 
         ins_idx = DVMBasicMethodBlock.start
         last_instru = None
         for DVMBasicMethodBlockInstruction in DVMBasicMethodBlock.get_instructions():
-            c_ins = {}
-            c_ins["idx"] = ins_idx
-            c_ins["name"] = DVMBasicMethodBlockInstruction.get_name()
-            c_ins["operands"] = DVMBasicMethodBlockInstruction.get_operands(
-                ins_idx)
-
-            c_ins[
-                "formatted_operands"] = DVMBasicMethodBlockInstruction.get_formatted_operands()
+            c_ins = {"idx": ins_idx,
+                     "name": DVMBasicMethodBlockInstruction.get_name(),
+                     "operands": DVMBasicMethodBlockInstruction.get_operands(ins_idx),
+                     "formatted_operands": DVMBasicMethodBlockInstruction.get_formatted_operands()}
 
             cblock["instructions"].append(c_ins)
 
-            if (DVMBasicMethodBlockInstruction.get_op_value() == 0x2b or DVMBasicMethodBlockInstruction.get_op_value() == 0x2c):
+            if (DVMBasicMethodBlockInstruction.get_op_value() == 0x2b or
+                DVMBasicMethodBlockInstruction.get_op_value() == 0x2c):
                 values = DVMBasicMethodBlock.get_special_ins(ins_idx)
                 cblock["info_next"] = values.get_values()
 
@@ -571,17 +552,20 @@ def method2json_direct(mx):
             if len(DVMBasicMethodBlock.childs) > 1:
                 cblock["info_bb"] = 1
 
-            if (last_instru.get_op_value() == 0x2b or last_instru.get_op_value() == 0x2c):
+            if (last_instru.get_op_value() == 0x2b or
+                last_instru.get_op_value() == 0x2c):
                 cblock["info_bb"] = 2
 
         cblock["Edge"] = []
         for DVMBasicMethodBlockChild in DVMBasicMethodBlock.childs:
             ok = False
             if DVMBasicMethodBlock.get_name() in hooks:
-                if DVMBasicMethodBlockChild[-1] in hooks[DVMBasicMethodBlock.get_name()]:
+                if DVMBasicMethodBlockChild[-1] in hooks[
+                    DVMBasicMethodBlock.get_name()
+                ]:
                     ok = True
-                    cblock["Edge"].append(
-                        hooks[DVMBasicMethodBlock.get_name()][0].get_name())
+                    cblock["Edge"].append(hooks[DVMBasicMethodBlock.get_name(
+                    )][0].get_name())
 
             if not ok:
                 cblock["Edge"].append(DVMBasicMethodBlockChild[-1].get_name())
@@ -651,17 +635,22 @@ class SVs(object):
         return self.__value.__str__()
 
 
-def object_to_str(obj):
+def object_to_bytes(obj):
+    """
+    Convert a object to a bytearray or call get_raw() of the object
+    if no useful type was found.
+    """
     if isinstance(obj, str):
-        return obj
+        return bytearray(obj, "UTF-8")
     elif isinstance(obj, bool):
-        return ""
+        return bytearray()
     elif isinstance(obj, int):
         return pack("<L", obj)
-    elif obj == None:
-        return ""
+    elif obj is None:
+        return bytearray()
+    elif isinstance(obj, bytearray):
+        return obj
     else:
-        # print type(obj), obj
         return obj.get_raw()
 
 
@@ -674,7 +663,7 @@ class MethodBC(object):
 class BuffHandle(object):
 
     def __init__(self, buff):
-        self.__buff = buff
+        self.__buff = bytearray(buff)
         self.__idx = 0
 
     def size(self):
@@ -691,16 +680,16 @@ class BuffHandle(object):
         return data
 
     def read_b(self, size):
-        return self.__buff[self.__idx: self.__idx + size]
+        return self.__buff[self.__idx:self.__idx + size]
 
     def read_at(self, offset, size):
-        return self.__buff[offset: offset + size]
+        return self.__buff[offset:offset + size]
 
     def read(self, size):
         if isinstance(size, SV):
             size = size.value
 
-        buff = self.__buff[self.__idx: self.__idx + size]
+        buff = self.__buff[self.__idx:self.__idx + size]
         self.__idx += size
 
         return buff
@@ -721,20 +710,20 @@ class Buff(object):
 class _Bytecode(object):
 
     def __init__(self, buff):
-        try:
-            import psyco
-            psyco.full()
-        except ImportError:
-            pass
-
-        self.__buff = buff
+        self.__buff = bytearray(buff)
         self.__idx = 0
 
+    def __getitem__(self, item):
+        return self.__buff[item]
+
+    def __len__(self):
+        return len(self.__buff)
+
     def read(self, size):
         if isinstance(size, SV):
             size = size.value
 
-        buff = self.__buff[self.__idx: self.__idx + size]
+        buff = self.__buff[self.__idx:self.__idx + size]
         self.__idx += size
 
         return buff
@@ -746,7 +735,7 @@ class _Bytecode(object):
         return self.__buff[off:]
 
     def read_b(self, size):
-        return self.__buff[self.__idx: self.__idx + size]
+        return self.__buff[self.__idx:self.__idx + size]
 
     def set_idx(self, idx):
         self.__idx = idx
@@ -771,38 +760,38 @@ class _Bytecode(object):
 
     def save(self, filename):
         buff = self._save()
-        with open(filename, "w") as fd:
+        with open(filename, "wb") as fd:
             fd.write(buff)
 
 
-def FormatClassToJava(input):
+def FormatClassToJava(i):
     """
-       Transoform a typical xml format class into java format
+       Transform a typical xml format class into java format
 
-       :param input: the input class name
+       :param i: the input class name
        :rtype: string
     """
-    return "L" + input.replace(".", "/") + ";"
+    return "L" + i.replace(".", "/") + ";"
 
 
-def FormatClassToPython(input):
-    i = input[:-1]
+def FormatClassToPython(i):
+    i = i[:-1]
     i = i.replace("/", "_")
     i = i.replace("$", "_")
 
     return i
 
 
-def FormatNameToPython(input):
-    i = input.replace("<", "")
+def FormatNameToPython(i):
+    i = i.replace("<", "")
     i = i.replace(">", "")
     i = i.replace("$", "_")
 
     return i
 
 
-def FormatDescriptorToPython(input):
-    i = input.replace("/", "_")
+def FormatDescriptorToPython(i):
+    i = i.replace("/", "_")
     i = i.replace(";", "")
     i = i.replace("[", "")
     i = i.replace("(", "")
